set -e

while [ $# -gt 0 ]; do
    case "$1" in
	-P) OPT_PLOT_DISABLE=1; shift;;
	-p) OPT_PLOT_ONLY=1; shift;;
	-X|--no-x11-plot) OPT_NO_X11=1; shift;;
    esac
done

PATH=$PWD/../_compiled/bin/:$PATH

COUNT=10000

error() {
    echo $1 >&2
    exit 1
}

sshgw() {
    local socket="$HOME/.ssh/cangw-connection"

    if [[ ! -S $socket ]]; then
        # Create master connection to speed up subsequenct command.
        # The ssh is put into background and the connection is closed
        # after 10 minutes)
	ssh -f -M -S $socket root@192.168.2.3 sleep 600 > /dev/null 2>&1
    fi
    ssh -x -a -S $socket root@192.168.2.3 "$@"
}

cleanupgw() {
    sshgw 'cangw -F'
}

_plot() {
    local testname=`basename $0 .sh`

    plot_cmds | sed -e "/set title/ s/[\"']\(.*\)[\"']/\"\1\\\\n($kvers)\"/" > plot.gp
    if [ -z "$OPT_NO_X11" ]; then
	echo "set terminal x11 enhanced; $(< plot.gp)" | gnuplot -persist
    fi
    I=''
    echo 'set terminal postscript color eps enhanced size 6cm,4cm lw 1 "Times-Roman" 10;' \
	'set lmargin 8;' \
	"$(< plot.gp)" | gnuplot > ${testname}$I.eps
    echo "set terminal postscript color eps enhanced;" \
	"$(< plot.gp)" | gnuplot | epstopdf --filter > ${testname}$I.pdf
    mkdir -p thumb
    convert -density 30  -gamma 0.5 -quality 90 -type Palette -depth 8 ${testname}$I.pdf thumb/${testname}$I.png
    convert -density 150 -gamma 0.7 -quality 90 -type Palette -depth 8 ${testname}$I.pdf ${testname}$I.png

}

create_dirs_and_links() {
    local test=$1
    local kver=$2

    local  d=results/by-kern/$kver/$test
    mkdir -p $d
    mkdir -p results/by-test/$test
    ln -sfT ../../${d#results/} results/by-test/$test/$kver
    echo $d
}

echo_plot() {
    plot=$1
    [[ "$_plot_separator" ]] && echo ", \\"
    _plot_separator=t
    echo -n "    " $plot
}

_run() {
    if [[ ! "$OPT_PLOT_ONLY" ]]; then
	kernel_versions=$(sshgw uname -r)
    else
	kernel_versions=$(ls results/by-kern)
    fi
    for kvers in $kernel_versions; do
	dir=$(create_dirs_and_links $(basename $0 .sh) $kvers)
	script=$PWD/$0
	cd $dir
	echo "Working directory: $dir"
	if [[ ! "$OPT_PLOT_ONLY" ]]; then
	    # Remove data from the last measurement
	    rm -rf *
            # Set can interfaces up
	    sshgw 'for i in 0 1; do ip link show dev can$i|grep -q UP || ip link set can$i up type can bitrate 1000000; done'
            # Delete all vcan interfaces
	    sshgw 'for dev in $(ip l|grep -o vcan[^:]\\+); do ip link del dev $dev; done'
	    # Reset priorities
	    sshgw 'chrt -p -f 50 `pidof irq/145-can0` > /dev/null || :'
	    sshgw 'chrt -p -f 50 `pidof irq/146-can1` > /dev/null || :'
	    sshgw 'chrt -p -f 49 `pidof sirq-net-rx/0` > /dev/null || :'
	    sshgw 'chrt -p -f 49 `pidof sirq-net-tx/0` > /dev/null || :'
	    cleanupgw
	    
	    main
	    cp $script .
	fi
	if [[ ! "$OPT_PLOT_DISABLE" ]]; then
	    _plot
	fi
    done
}

test_end() {
    test_end_called=t
    exit_ok=
    _run
    exit_ok=t
}

_myexit() {
    code=$?
    cmd=$BASH_COMMAND
    if [[ ! "$test_end_called" ]]; then
	test_end_called=t
	error "bug in the test script: No test_end called"
    fi
    if ! test -n "$exit_ok"; then
	error "FATAL: Command '$cmd' exited with code $code"
    fi
}

trap '_myexit' EXIT
